'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = identifierReversePlugin;

var _core = require('@babel/core');

function constructMemberExpression(chain) {
    if (chain.length === 1) {
        return _core.types.identifier(chain[0]);
    }
    const [first, ...rest] = chain;
    return _core.types.memberExpression(constructMemberExpression(rest), _core.types.identifier(first));
}

function constructLogicalExpression(chain) {
    const right = _core.types.binaryExpression('!=', constructMemberExpression(chain), _core.types.nullLiteral());
    if (chain.length === 1) {
        return right;
    }
    const [_, ...rest] = chain;
    return _core.types.logicalExpression('&&', constructLogicalExpression(rest), right);
}

function isOcExpression(mExpr) {
    return _core.types.isCallExpression(mExpr) && _core.types.isIdentifier(mExpr.callee) && mExpr.callee.name === 'oc' && mExpr.arguments.length === 1 && _core.types.isIdentifier(mExpr.arguments[0]);
}

function void0() {
    return _core.types.unaryExpression('void', _core.types.numericLiteral(0));
}

function identifierReversePlugin() {
    return {
        name: 'babel-plugin-transform-optchain',
        visitor: {
            CallExpression(path) {
                if (_core.types.isMemberExpression(path.node.callee) && path.node.arguments.length <= 1) {
                    let mExpr = path.node.callee;
                    let chain = [];
                    while (_core.types.isMemberExpression(mExpr)) {
                        chain.push(mExpr.property.name);
                        mExpr = mExpr.object;
                    }
                    if (isOcExpression(mExpr)) {
                        chain.push(mExpr.arguments[0].name);
                        path.replaceWith(_core.types.conditionalExpression(constructLogicalExpression(chain), constructMemberExpression(chain), path.node.arguments.length === 1 ? path.node.arguments[0] : void0()));
                    }
                } else if (isOcExpression(path.node)) {
                    throw new Error('[optchain-transform] Can\'t handle partial expressions.');
                }
            },
            ImportDeclaration(path) {
                if (_core.types.isLiteral(path.node.source) && path.node.source.value === 'ts-optchain') {
                    path.remove();
                }
            }
        }
    };
}
